import requests
import os
import json
import logging

# Setup logging
logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(levelname)s - %(message)s')

class KoreaInvestmentAPI:
    def __init__(self):
        self.appkey = os.getenv('MY_APP')
        self.appsecret = os.getenv('MY_SEC')
        self.user_id = os.getenv('USER_ID')
        self.base_url = os.getenv('PROD_URL')

        # Token 읽기
        token_path = os.path.join('C:\\Users\\user\\KIS\\config', 'token.json')
        if not os.path.exists(token_path):
            raise ValueError(f"Token file does not exist: {token_path}")

        with open(token_path, 'r') as file:
            token_data = json.load(file)
            self.access_token = token_data.get('token')

        if not all([self.appkey, self.appsecret, self.access_token, self.user_id, self.base_url]):
            missing_vars = [var for var in ['MY_APP', 'MY_SEC', 'ACCESS_TOKEN', 'USER_ID', 'PROD_URL'] if os.getenv(var) is None]
            raise ValueError(f"Some required environment variables are missing: {', '.join(missing_vars)}")

    def get_search_conditions(self):
        path = "uapi/domestic-stock/v1/quotations/psearch-title"
        url = f"{self.base_url}/{path}"
        headers = {
            "Content-Type": "application/json",
            "authorization": f"Bearer {self.access_token}",
            "appkey": self.appkey,
            "appsecret": self.appsecret,
            "tr_id": "HHKST03900300",
            "custtype": "P"
        }
        params = {
            "user_id": self.user_id
        }

        try:
            response = requests.get(url, headers=headers, params=params)
            response.raise_for_status()
            logging.info("Search conditions response: %s", response.json())
            return response.json()
        except requests.exceptions.HTTPError as http_err:
            logging.error(f"HTTP error occurred: {http_err}")
            logging.error(f"Response content: {response.content}")
        except Exception as err:
            logging.error(f"Other error occurred: {err}")

    def get_search_results(self, seq):
        path = "uapi/domestic-stock/v1/quotations/psearch-result"
        url = f"{self.base_url}/{path}"
        headers = {
            "Content-Type": "application/json",
            "authorization": f"Bearer {self.access_token}",
            "appkey": self.appkey,
            "appsecret": self.appsecret,
            "tr_id": "HHKST03900400",
            "custtype": "P"
        }
        params = {
            "user_id": self.user_id,
            "seq": seq
        }

        try:
            response = requests.get(url, headers=headers, params=params)
            response.raise_for_status()
            logging.info("Search results response: %s", response.json())
            return response.json()
        except requests.exceptions.HTTPError as http_err:
            logging.error(f"HTTP error occurred: {http_err}")
            logging.error(f"Response content: {response.content}")
        except Exception as err:
            logging.error(f"Other error occurred: {err}")
