import pandas as pd
import numpy as np
import os


def fetch_stock_data(file_path='historical_stock_data.csv'):
    print(f"Fetching stock data from: {file_path}")
    if not os.path.exists(file_path):
        print(f"File not found: {file_path}")
        return pd.DataFrame()
    
    try:
        df = pd.read_csv(file_path)
        print("Raw data from file:")
        print(df)  # 파일에서 읽은 데이터 출력
        if df.empty:
            print("DataFrame is empty")
        else:
            print(f"Data fetched: {df.head()}")
        return df
    except pd.errors.EmptyDataError as e:
        print(f"Empty data error: {e}")
        return pd.DataFrame()
    except Exception as e:
        print(f"Error fetching stock data: {e}")
        return pd.DataFrame()
    
def calculate_technical_indicators(df):
    df['SMA_50'] = df['price'].rolling(window=50).mean()
    df['SMA_200'] = df['price'].rolling(window=200).mean()
    df['EMA_50'] = df['price'].ewm(span=50, adjust=False).mean()
    df['EMA_200'] = df['price'].ewm(span=200, adjust=False).mean()
    df['MACD'] = df['EMA_50'] - df['EMA_200']
    df['RSI'] = calculate_rsi(df['price'])
    return df

def calculate_rsi(series, period=14):
    delta = series.diff()
    gain = (delta.where(delta > 0, 0)).rolling(window=period).mean()
    loss = (-delta.where(delta < 0, 0)).rolling(window=period).mean()
    rs = gain / loss
    return 100 - (100 / (1 + rs))

def feature_selection(df):
    df = df.dropna()
    X = df[['price', 'SMA_50', 'SMA_200', 'EMA_50', 'EMA_200', 'MACD', 'RSI']]
    y = df['price'].shift(-1) > df['price']
    
    from sklearn.feature_selection import SelectKBest, f_classif
    selector = SelectKBest(score_func=f_classif, k='all')
    selector.fit(X, y)
    scores = selector.scores_
    feature_scores = pd.DataFrame({'Feature': X.columns, 'Score': scores}).sort_values(by='Score', ascending=False)
    return feature_scores

# 임시 데이터 파일 생성 코드
# 임시 데이터 파일 생성 코드
def create_temp_data():
    data = {
        'date': ['2023-01-01', '2023-01-02', '2023-01-03', '2023-01-04', '2023-01-05'],
        'price': [100, 105, 102, 110, 115]
    }

    df = pd.DataFrame(data)
    df.to_csv('historical_stock_data.csv', index=False)
    print("Temporary historical_stock_data.csv created.")
    print("Contents of historical_stock_data.csv:")
    print(df)

if __name__ == "__main__":
    data = {
        'date': ['2023-01-01', '2023-01-02', '2023-01-03', '2023-01-04', '2023-01-05'],
        'price': [100, 105, 102, 110, 115]
    }

    df = pd.DataFrame(data)
    df.to_csv('historical_stock_data.csv', index=False)
    print("Temporary historical_stock_data.csv created.")
    print("Contents of historical_stock_data.csv:")
    print(df)
    
    # 데이터 파일 읽기 테스트
    fetch_stock_data('historical_stock_data.csv')