import backtrader as bt
import numpy as np
import pandas as pd
from sklearn.ensemble import RandomForestClassifier

class VWAP(bt.Indicator):
    lines = ('vwap',)
    params = (
        ('period', 15),
    )

    def __init__(self):
        self.addminperiod(self.params.period)
        self.plotinfo.plotyhlines = [0.0]

    def next(self):
        typical_price = (self.data.close[0] + self.data.high[0] + self.data.low[0]) / 3
        if len(self) == 1:
            self.lines.vwap[0] = typical_price
        else:
            cumulative_price_volume = (self.lines.vwap[-1] * (len(self) - 1) + typical_price * self.data.volume[0])
            cumulative_volume = (len(self) - 1) * self.data.volume[0] + self.data.volume[0]
            self.lines.vwap[0] = cumulative_price_volume / cumulative_volume

class AdvancedAdaptiveStrategy(bt.Strategy):
    params = (
        ('lookback', 15),  # VWAP 계산을 위한 기간
        ('rsi_period', 14),  # RSI 계산을 위한 기간
        ('macd1', 12),  # MACD 계산을 위한 첫 번째 EMA 기간
        ('macd2', 26),  # MACD 계산을 위한 두 번째 EMA 기간
        ('macdsig', 9),  # MACD 시그널 기간
        ('rsi_overbought', 70),  # RSI 과매수 조건
        ('rsi_oversold', 30),  # RSI 과매도 조건
        ('stop_loss', 0.02),  # 손절매 기준 (2%)
        ('take_profit', 0.04),  # 이익 실현 기준 (4%)
        ('volatility_window', 20)  # 변동성 계산을 위한 기간
    )

    def __init__(self):
        self.vwap = VWAP(self.data, period=self.params.lookback)
        self.rsi = bt.indicators.RelativeStrengthIndex(self.data, period=self.params.rsi_period)
        self.macd = bt.indicators.MACD(self.data,
                                       period_me1=self.params.macd1,
                                       period_me2=self.params.macd2,
                                       period_signal=self.params.macdsig)
        self.bollinger = bt.indicators.BollingerBands(self.data)
        self.atr = bt.indicators.AverageTrueRange(self.data)
        
        self.model = RandomForestClassifier()
        self.train_model()

    def train_model(self):
        # 모델 학습을 위한 데이터 준비
        close = np.array([self.data.close[i] for i in range(len(self.data))])
        volume = np.array([self.data.volume[i] for i in range(len(self.data))])
        high = np.array([self.data.high[i] for i in range(len(self.data))])
        low = np.array([self.data.low[i] for i in range(len(self.data))])

        # Create DataFrame
        data = pd.DataFrame({'close': close, 'volume': volume, 'high': high, 'low': low})
        features = data[['close', 'volume', 'high', 'low']].pct_change()

        # VWAP 계산 추가
        typical_price = (data['close'] + data['high'] + data['low']) / 3
        features['vwap'] = (typical_price * data['volume']).cumsum() / data['volume'].cumsum()

        # RSI 계산 추가
        features['rsi'] = data['close'].rolling(window=self.params.rsi_period).apply(self.rsi_calculate)

        # MACD 계산 추가
        macd_short_period = 12
        macd_long_period = 26
        macd_signal_period = 9
        macd = data['close'].ewm(span=macd_short_period, adjust=False).mean() - data['close'].ewm(span=macd_long_period, adjust=False).mean()
        signal = macd.ewm(span=macd_signal_period, adjust=False).mean()
        features['macd'] = macd - signal

        # Bollinger Bands 계산 추가
        bollinger_window = 20
        rolling_mean = data['close'].rolling(window=bollinger_window).mean()
        rolling_std = data['close'].rolling(window=bollinger_window).std()
        features['bollinger_high'] = rolling_mean + (rolling_std * 2)
        features['bollinger_low'] = rolling_mean - (rolling_std * 2)

        # Drop NaNs introduced by these calculations
        features = features.dropna()
        print(f"Features after dropping NaNs:\n{features}")

        # Label generation: next day's close price > today's close price
        labels = (data['close'].shift(-1) > data['close']).astype(int)
        labels = labels.loc[features.index]
        print(f"Labels after alignment:\n{labels}")

        # Check if features or labels are empty
        if features.empty or labels.empty:
            print("Error: Features or labels are empty. Cannot train the model.")
            return

        # 모델 학습
        self.model.fit(features, labels)

    def rsi_calculate(self, series):
        delta = series.diff()
        gain = (delta.where(delta > 0, 0)).mean()
        loss = (-delta.where(delta < 0, 0)).mean()
        rs = gain / loss
        return 100 - (100 / (1 + rs))
