import pandas as pd
import numpy as np
import backtrader as bt
from sklearn.ensemble import RandomForestClassifier
from sklearn.model_selection import train_test_split
from sklearn.metrics import accuracy_score, classification_report

# rsi_calculate 함수 정의
def rsi_calculate(series):
    delta = series.diff()
    gain = delta.where(delta > 0, 0).mean()
    loss = -delta.where(delta < 0, 0).mean()
    rs = gain / loss
    return 100 - (100 / (1 + rs))

# 실제 데이터 로드 (예: real_data.csv)
df = pd.read_csv('real_data.csv', parse_dates=['datetime'], index_col='datetime')

# 거래 가능한 시간 필터링 (예: 09:00 ~ 15:00)
market_hours = df.between_time('09:00', '15:00')
print(f"Filtered market hours data:\n{market_hours}")

# 지표 계산 예시
features = market_hours[['close', 'volume', 'high', 'low']].pct_change().dropna()
print(f"Features after pct_change:\n{features}")

# NaN 제거 후 지표 계산
typical_price = (market_hours['close'] + market_hours['high'] + market_hours['low']) / 3
features['vwap'] = (typical_price * market_hours['volume']).cumsum() / market_hours['volume'].cumsum()

# RSI 계산
features['rsi'] = market_hours['close'].rolling(window=14).apply(rsi_calculate).dropna()

# MACD 계산
macd_short_period = 12
macd_long_period = 26
macd_signal_period = 9
macd = market_hours['close'].ewm(span=macd_short_period, adjust=False).mean() - market_hours['close'].ewm(span=macd_long_period, adjust=False).mean()
signal = macd.ewm(span=macd_signal_period, adjust=False).mean()
features['macd'] = macd - signal

# Bollinger Bands 계산
bollinger_window = 20
rolling_mean = market_hours['close'].rolling(window=bollinger_window).mean()
rolling_std = market_hours['close'].rolling(window=bollinger_window).std()
features['bollinger_high'] = rolling_mean + (rolling_std * 2)
features['bollinger_low'] = rolling_mean - (rolling_std * 2)

# NaN 제거
features = features.dropna()
print(f"Features after dropping NaNs:\n{features}")

# 라벨 생성
labels = (market_hours['close'].shift(-1) > market_hours['close']).astype(int)
labels = labels.loc[features.index]
labels = labels.dropna()
print(f"Labels after alignment:\n{labels}")

# 특징과 라벨 정렬
features = features.loc[labels.index]
print(f"Features shape: {features.shape}")
print(f"Labels shape: {labels.shape}")

# 데이터 분할: 훈련 데이터와 테스트 데이터
X_train, X_test, y_train, y_test = train_test_split(features, labels, test_size=0.2, random_state=42)

# 모델 훈련
model = RandomForestClassifier()
model.fit(X_train, y_train)

# 예측
y_pred = model.predict(X_test)

# 모델 평가
accuracy = accuracy_score(y_test, y_pred)
report = classification_report(y_test, y_pred)

print(f"Accuracy: {accuracy}")
print(f"Classification Report:\n{report}")

# 특징 중요도 출력
importances = model.feature_importances_
feature_names = features.columns
feature_importance_df = pd.DataFrame({'feature': feature_names, 'importance': importances})
feature_importance_df = feature_importance_df.sort_values(by='importance', ascending=False)
print(f"Feature Importances:\n{feature_importance_df}")

# backtrader 데이터 피드로 변환
data = bt.feeds.PandasData(dataname=df)

class PredictStrategy(bt.Strategy):
    def __init__(self):
        self.data_close = self.datas[0].close
        self.model = model  # 이미 훈련된 모델

    def next(self):
        if len(self.data) < 26:  # 최소 데이터 길이 확인 (MACD)
            return

        # 특징 계산 (실제 데이터에서 지표 계산)
        features = {
            'close': self.data.close[0],
            'volume': self.data.volume[0],
            'high': self.data.high[0],
            'low': self.data.low[0],
            'vwap': (self.data.close[0] + self.data.high[0] + self.data.low[0]) / 3,
            'rsi': bt.indicators.RSI(self.data, period=14)[0] if len(self.data) >= 14 else np.nan,
            'macd': bt.indicators.MACD(self.data).macd[0] if len(self.data) >= 26 else np.nan,
            'bollinger_high': bt.indicators.BollingerBands(self.data).top[0] if len(self.data) >= 20 else np.nan,
            'bollinger_low': bt.indicators.BollingerBands(self.data).bot[0] if len(self.data) >= 20 else np.nan
        }

        feature_values = np.array(list(features.values())).reshape(1, -1)

        # NaN 값이 포함된 경우 예측하지 않음
        if np.isnan(feature_values).any():
            return

        # 모델 예측
        prediction = self.model.predict(feature_values)

        # 매수/매도 결정
        if prediction == 1:
            self.buy(size=100)
        elif prediction == 0:
            self.sell(size=100)

# Cerebro 설정
cerebro = bt.Cerebro()
cerebro.adddata(data)
cerebro.addstrategy(PredictStrategy)
cerebro.run()
