import time
import copy
import yaml
import requests
import json
import os
from collections import namedtuple
from datetime import datetime
from dotenv import load_dotenv

# Load .env file
load_dotenv()

# Load configurations from .env
MY_AGENT = os.getenv('MY_AGENT')
MY_APP = os.getenv('MY_APP')
MY_SEC = os.getenv('MY_SEC')
MY_ACCT_STOCK = os.getenv('MY_ACCT_STOCK')
MY_ACCT_FUTURE = os.getenv('MY_ACCT_FUTURE')
MY_PROD = os.getenv('MY_PROD')
PROD_URL = os.getenv('PROD_URL')

# Use user home directory or current working directory
config_root = os.path.expanduser('~\\KIS\\config\\')  # Directory where token files are stored.
# Or use current working directory
# config_root = os.getcwd() + '\\KIS\\config\\'

token_file = os.path.join(config_root, 'token.json')  # Fixed token file name

# Ensure the directory exists
os.makedirs(config_root, exist_ok=True)

# Initialize global variables
KISEnv = namedtuple('KISEnv', ['my_app', 'my_sec', 'my_acct', 'my_prod', 'my_token', 'my_url'])
_last_auth_time = datetime.now()
_autoReAuth = False
_DEBUG = False
_isPaper = False
_TRENV = None

# Default headers
_base_headers = {
    "Content-Type": "application/json",
    "Accept": "text/plain",
    "charset": "UTF-8",
    'User-Agent': MY_AGENT
}

# Save the token and its expiry date to a file.
def save_token(my_token, my_expired):
    valid_date = datetime.strptime(my_expired, '%Y-%m-%d %H:%M:%S')
    with open(token_file, 'w', encoding='utf-8') as f:
        json.dump({'token': my_token, 'valid-date': valid_date.strftime('%Y-%m-%d %H:%M:%S')}, f)
    print(f"Token saved to {token_file}")

# Read the token from the file.
def read_token():
    try:
        with open(token_file, 'r', encoding='UTF-8') as f:
            tkg_tmp = json.load(f)

        exp_dt = datetime.strptime(tkg_tmp['valid-date'], '%Y-%m-%d %H:%M:%S')
        now_dt = datetime.now()

        if exp_dt > now_dt:
            return tkg_tmp['token']
        else:
            return None
    except Exception as e:
        return None

# Check token validity and re-authenticate if needed.
def _getBaseHeader():
    if _autoReAuth:
        reAuth()
    return copy.deepcopy(_base_headers)

# Set the environment configuration based on the JSON configuration.
def _setTRENV(cfg):
    global _TRENV
    _TRENV = KISEnv(
        my_app=cfg['my_app'],
        my_sec=cfg['my_sec'],
        my_acct=cfg['my_acct'],
        my_prod=cfg['my_prod'],
        my_token=cfg['my_token'],
        my_url=cfg['my_url']
    )

# Function to handle the authentication process.
def auth(svr='prod', product=MY_PROD, url=None):
    p = {
        "grant_type": "client_credentials",
        "appkey": MY_APP,
        "appsecret": MY_SEC
    }

    saved_token = read_token()
    if saved_token is None:
        url = f'{PROD_URL}/oauth2/tokenP'
        res = requests.post(url, data=json.dumps(p), headers=_getBaseHeader())
        rescode = res.status_code
        if rescode == 200:
            response_data = res.json()
            my_token = response_data['access_token']
            my_expired = response_data['access_token_token_expired']
            save_token(my_token, my_expired)
        else:
            print('Get Authentication token fail!\nYou have to restart your app!!!')
            return
    else:
        my_token = saved_token

    changeTREnv(f"Bearer {my_token}", svr, product)
    _base_headers["authorization"] = _TRENV.my_token
    _base_headers["appkey"] = _TRENV.my_app
    _base_headers["appsecret"] = _TRENV.my_sec

    global _last_auth_time
    _last_auth_time = datetime.now()

    if _DEBUG:
        print(f'[{_last_auth_time}] => get AUTH Key completed!')

# Change the environment settings based on the token and server type.
def changeTREnv(token_key, svr='prod', product=MY_PROD):
    cfg = {
        'my_app': MY_APP,
        'my_sec': MY_SEC,
        'my_acct': MY_ACCT_STOCK if product == '01' else MY_ACCT_FUTURE,
        'my_prod': product,
        'my_token': token_key,
        'my_url': PROD_URL
    }

    global _isPaper
    _isPaper = (svr != 'prod')

    _setTRENV(cfg)

def reAuth(svr='prod', product=MY_PROD):
    n2 = datetime.now()
    if (n2 - _last_auth_time).seconds >= 86400:
        auth(svr, product)

def getEnv():
    return _cfg

def getTREnv():
    return _TRENV

def set_order_hash_key(h, p):
    url = f"{getTREnv().my_url}/uapi/hashkey"
    res = requests.post(url, data=json.dumps(p), headers=h)
    rescode = res.status_code
    if rescode == 200:
        h['hashkey'] = res.json()['HASH']
    else:
        print("Error:", rescode)

class APIResp:
    def __init__(self, resp):
        self._rescode = resp.status_code
        self._resp = resp
        self._header = self._setHeader()
        self._body = self._setBody()
        self._err_code = self._body.msg_cd
        self._err_message = self._body.msg1

    def getResCode(self):
        return self._rescode

    def _setHeader(self):
        fld = {x: self._resp.headers.get(x) for x in self._resp.headers.keys() if x.islower()}
        return namedtuple('header', fld.keys())(**fld)

    def _setBody(self):
        return namedtuple('body', self._resp.json().keys())(**self._resp.json())

    def getHeader(self):
        return self._header

    def getBody(self):
        return self._body

    def getResponse(self):
        return self._resp

    def isOK(self):
        try:
            return self.getBody().rt_cd == '0'
        except:
            return False

    def getErrorCode(self):
        return self._err_code

    def getErrorMessage(self):
        return self._err_message

    def printAll(self):
        print("<Header>")
        for x in self.getHeader()._fields:
            print(f'\t-{x}: {getattr(self.getHeader(), x)}')
        print("<Body>")
        for x in self.getBody()._fields:
            print(f'\t-{x}: {getattr(self.getBody(), x)}')

    def printError(self, url):
        print('-------------------------------\nError in response: ', self.getResCode(), ' url=', url)
        print('rt_cd : ', self.getBody().rt_cd, '/ msg_cd : ', self.getErrorCode(), '/ msg1 : ', self.getErrorMessage())
        print('-------------------------------')

def _url_fetch(api_url, ptr_id, tr_cont, params, appendHeaders=None, postFlag=False, hashFlag=True):
    url = f"{getTREnv().my_url}{api_url}"
    headers = _getBaseHeader()

    tr_id = ptr_id
    if ptr_id[0] in ('T', 'J', 'C') and isPaperTrading():
        tr_id = 'V' + ptr_id[1:]

    headers["tr_id"] = tr_id
    headers["custtype"] = "P"
    headers["tr_cont"] = tr_cont

    if appendHeaders:
        for k, v in appendHeaders.items():
            headers[k] = v

    if _DEBUG:
        print("< Sending Info >")
        print(f"URL: {url}, TR: {tr_id}")
        print(f"<header>\n{headers}")
        print(f"<body>\n{params}")

    res = requests.post(url, headers=headers, data=json.dumps(params)) if postFlag else requests.get(url, headers=headers, params=params)
    if res.status_code == 200:
        ar = APIResp(res)
        if _DEBUG:
            ar.printAll()
        return ar
    else:
        print("Error Code : " + str(res.status_code) + " | " + res.text)
        return None

# Main authentication call
auth()
